/*

    FIR Decimation class, header
    Copyright (C) 2000-2001 Jussi Laako

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#include <Alloc.hh>

#include "dsp/DSPConfig.hh"
#include "dsp/DSPOp.hh"
#ifndef DSP_USE_DEQUE
#include "dsp/ReBuffer.hh"
#else
#include "dsp/ReBuffer2.hh"
#endif


#ifndef FIRDECIMATOR_HH
    #define FIRDECIMATOR_HH


    /**
        FIR decimation filter class implementation.

        Data is is filtered using FIR filter and decimated by factor 2, 4 or 8.
    */
    class clFIRDecimator
    {
            float fGain;  ///< Filter gain
            double dGain;  ///< Filter gain
            long lFactor;  ///< Decimation factor
            clDSPAlloc DecBuf;
            clDSPOp DSP;
            clDSPOp FIR;
            #ifndef DSP_USE_DEQUE
            clReBuffer InBuf;
            #else
            clReBuffer2 InBuf;
            #endif
        public:
            clFIRDecimator ();
            ~clFIRDecimator ();
            /**
                Initialize decimator, decimation factor must be power of two.
                You can re-initialize decimator without uninitializing first.

                The NULL pointer is used select correct overloaded function
                matching input datatype.

                \param DecFact Decimation factor
                \param NullPtr NULL pointer
                \param HighPass High-pass filter?
            */
            bool Initialize (long, const float *, bool = false);
            /// \overload
            bool Initialize (long, const double *, bool = false);
            /**
                Uninitialize decimator
            */
            void Uninitialize ();
            /**
                Feed data into filter.

                \param SrcData Source data
                \param SrcCount Source data count
            */
            void Put (const float *, long);
            /// \overload
            void Put (const double *, long);
            /**
                Get data from filter.

                \param DestData Destination buffer
                \param DestCount Number of samples to fetch
                \return Success
            */
            bool Get (float *, long);
            /// \overload
            bool Get (double *, long);
    };

#endif

